<?php

/**
 * Copyright 2017 Projectmoon, SRL.
 *
 * You are hereby granted a non-exclusive, worldwide, royalty-free license to
 * use, copy, modify, and distribute this software in source code or binary
 * form for use in connection with the web services and APIs provided by
 * Storeden. More information about policies can be found here:
 * [https://developers.storeden.com/docs/apps/policy]
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 */

if (!function_exists('curl_init')) {
  throw new Exception('Storeden needs the CURL PHP extension.');
}
if (!function_exists('json_decode')) {
  throw new Exception('Storeden needs the JSON PHP extension.');
}

class StoredenConnectSDK {

    private $version = '1.1b';

    private $base_path = NULL;

    private $api_version = NULL;

    private $key = NULL;

    private $exchange = NULL;

    private $curl_opts = array(
      CURLOPT_CONNECTTIMEOUT => 30,
      CURLOPT_RETURNTRANSFER => true,
      CURLOPT_TIMEOUT        => 120,
      CURLOPT_SSLVERSION     => CURL_SSLVERSION_DEFAULT,
      CURLOPT_USERAGENT      => 'storeden-php-1.1b',
    );

    public function __construct($options){

        if(!isset($options['key'])){
            throw new Exception('StoredenConnectSDK need your Connect key');
        }

        if(!isset($options['key'])){
            throw new Exception('StoredenConnectSDK need your Connect exchange key');
        }

        $this->key = (string)$options['key'];

        $this->exchange = (string)$options['exchange'];

        $curl_options = $this->curl_opts;
        $curl_options[CURLOPT_HTTPHEADER] = array(
            'key: '.$this->key,
            'exchange: '.$this->exchange,
            'Expect: '
        );

        $this->curl_opts = $curl_options;

        if($options['api_version'] != NULL ){

            $this->__validate_api_version($options['api_version']);

            $this->api_version = (string)$options['api_version'];
            $this->base_path = 'https://connect.storeden.com/'.$this->api_version;

        }else{
            $this->api_version = $this->__obtain_api_version();
        }

    }

    private function __validate_api_version($version){

        $curl = curl_init();
        $curl_opts = $this->curl_opts;
        $curl_opts[CURLOPT_URL] = 'https://connect.storeden.com/'.$version.'/validate';

        curl_setopt_array($curl, $curl_opts);

        $curl_response = curl_exec($curl);

        $info = $this->__parse_response($curl_response);

        if($info->status != 'active'){
           trigger_error('Storeden Connect API Version '.$version.' has been deprecated. Please update your API version', E_USER_WARNING);
        }

    }

    /**
     * Check what is the best version of API to connect with.
     * Unversioned API will be always availabe to connect to /platform
     **/

    private function __obtain_api_version(){

        $curl = curl_init();
        $curl_opts = $this->curl_opts;
        $curl_opts[CURLOPT_URL] = 'https://connect.storeden.com/unversioned/platform';
        $curl_opts[CURLOPT_VERBOSE] = true;
        curl_setopt_array($curl, $curl_opts);

        $curl_response = curl_exec($curl);

        $info = $this->__parse_response($curl_response);

        $this->api_version = $info->api->recommended;

        $this->base_path = 'https://connect.storeden.com/'.$this->api_version;

    }

    /**
     * Utility to validate and return correct response format.
     * If call fail return a generic error
     **/

    private function __parse_response($curl_output){

        if($curl_output == NULL)
            throw new Exception('Invalid response from API');

        $content = json_decode($curl_output);

        return $content;

    }

    /**
     * Class method to dialog with API
     * This method will be extended with a better error reporting.
     **/

    public function api($query, $type = 'GET', $params = array(), $callback = NULL, $forbid_reuse = true){

        $curl = curl_init();
        $curl_opts = $this->curl_opts;

        if($type != 'GET'){
            $curl_opts[CURLOPT_CUSTOMREQUEST] =  $type;
            $curl_opts[CURLOPT_POSTFIELDS] =  http_build_query($params);
        }

        $curl_opts[CURLOPT_URL] = $this->base_path.$query;

        curl_setopt_array($curl, $curl_opts);

        $curl_response = curl_exec($curl);

        curl_close($curl);

        if ($callback == NULL){
            return $this->__parse_response($curl_response);
        }else{
            return call_user_func($callback, $this->__parse_response($curl_response));
        }

    }

    public function printConnectionStatus(){

        $connection_info = $this->api('/connection');

        if( php_sapi_name() == 'cli' ){
            echo PHP_EOL.'** STOREDEN SDK '.$this->version.' ** '.PHP_EOL;
            echo 'API Ratio Usage: '.$connection_info->current_usage->percentage.' '.PHP_EOL;
        }

    }

    public function retriveImage($url){
        if(!($url > '')){
    		return false;
    	}
    	$url = str_replace('"','',$url);

    	$ch = curl_init();
    	curl_setopt($ch, CURLOPT_AUTOREFERER, TRUE);
    	curl_setopt($ch, CURLOPT_HEADER, 0);
    	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    	curl_setopt($ch, CURLOPT_URL, $url);
    	curl_setopt($ch,CURLOPT_TIMEOUT,10);
    	curl_setopt($ch, CURLOPT_FOLLOWLOCATION, TRUE);
    	$content = curl_exec($ch);
    	curl_close($ch);

        return $content;
    }

}
